//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy;

import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.media.AudioManager;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.DisplayMetrics;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.Toast;
import aschiefer.bouncy.model.GameModel;
import aschiefer.bouncy.model.PongGameModel;
import aschiefer.bouncy.renderer.GameRenderer;
import aschiefer.bouncy.renderer.PongGameRenderer;

public class GameActivity extends Activity {
	private GLSurfaceView _gl_view;
	private GameModel _model;
	private GameRenderer _renderer;
   
	static final int DIALOG_PAUSED_ID = 0;
   
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.game_overlay);
        ViewGroup layout = (ViewGroup) findViewById(R.id.RelativeLayout01);
        findViewById(R.id.ButtonGameStart).setOnClickListener(
				new OnClickListener() {
					@Override
					public void onClick(View view) {
						view.setVisibility(View.INVISIBLE);
						if (_model.isRunning()) {
							_model.stop();
						}
						_model.start();
						Toast.makeText(GameActivity.this, R.string.ready, Toast.LENGTH_SHORT).show();
					}
				});
        setVolumeControlStream(AudioManager.STREAM_MUSIC);
        //Get fullscreen resolution
		DisplayMetrics metrics = new DisplayMetrics();
		getWindowManager().getDefaultDisplay().getMetrics(metrics);
        _gl_view = new GLSurfaceView(this);
        _model = new PongGameModel(PreferenceManager.getDefaultSharedPreferences(this),
        		metrics.widthPixels, metrics.heightPixels);
        _renderer = new PongGameRenderer(this);
        _renderer.setLayout(layout);
        _renderer.setGameModel(_model);
        _gl_view.setRenderer(_renderer);
        layout.addView(_gl_view, 0);
        
    }
    
    @Override
    public void onResume() {
    	super.onResume();
    	_gl_view.onResume();
    	_renderer.setGameModel(_model);
    	showDialog(DIALOG_PAUSED_ID);
    }
    
    @Override
    public void onPause() {
    	super.onPause();
    	_gl_view.onPause();
    	_model.pause();
    }  
    
    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
    	super.dispatchKeyEvent(event);
    	_model.getInputSystem().handleKeyEvent(event);
    	return false;
    }
    
    @Override
    public boolean dispatchTouchEvent(MotionEvent event) {
    	super.dispatchTouchEvent(event);
    	_model.getInputSystem().handleTouchEvent(event);
    	return false;
    }
    
    @Override
    public boolean dispatchTrackballEvent(MotionEvent event) {
    	super.dispatchTrackballEvent(event);
    	_model.getInputSystem().handleTrackballEvent(event);
		return false;
    }
    
    
    public boolean onKeyDown(int keyCode, KeyEvent event) {
    	if (keyCode == KeyEvent.KEYCODE_MENU) {
    		_model.pause();
    		System.gc();
    		showDialog(DIALOG_PAUSED_ID);
    	}
    	return false;
    }
    
    protected Dialog onCreateDialog(int id) {
    	if (id == DIALOG_PAUSED_ID) {    		
    		Dialog dialog = new Dialog(GameActivity.this);
    		dialog.setContentView(R.layout.main_menu);
    		dialog.setVolumeControlStream(AudioManager.STREAM_MUSIC);
    		dialog.setTitle(R.string.app_name);
    		dialog.findViewById(R.id.ButtonStart).setOnClickListener(
    				new OnClickListener() {
						@Override
						public void onClick(View arg0) {
							GameActivity.this.findViewById(R.id.ButtonGameStart).setVisibility(View.INVISIBLE);
							if (_model.isRunning()) {
								_model.stop();
							}
							_model.start();
							Toast.makeText(GameActivity.this, R.string.ready, Toast.LENGTH_SHORT).show();
							GameActivity.this.dismissDialog(DIALOG_PAUSED_ID);
						}
					});
    		dialog.findViewById(R.id.ButtonResume).setOnClickListener(
    				new OnClickListener() {
						@Override
						public void onClick(View arg0) {
							_model.resume();
							GameActivity.this.dismissDialog(DIALOG_PAUSED_ID);
						}
					});
    		dialog.findViewById(R.id.ButtonSettings).setOnClickListener(
    				new OnClickListener() {
						@Override
						public void onClick(View arg0) {
							Intent i = new Intent(GameActivity.this, SettingsActivity.class);
							startActivity(i);
						}
					});
    		return dialog;
    	}
    	return null;
    }
    
    protected void onPrepareDialog(int id, Dialog dialog) {
    	if (id == DIALOG_PAUSED_ID) {
    		dialog.findViewById(R.id.ButtonResume).setEnabled(_model.isRunning());
    	}
    }
    
}