//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.model;

import android.graphics.PointF;

public class PongBallModel {
	PongGameModel _model;
	//Position
	protected float _x;
	protected float _y;
	//Velocity
	protected float _vx;
	protected float _vy;
	//Size
	protected float _size;
	protected float _half_size;
	
	protected float _max_velocity;
	
	public PongBallModel(PongGameModel model, float x, float y, float size, float vx, float vy) {
		_model = model;
		_x = x;
		_y = y;
		_vx = vx;
		_vy = vy;
		_size = size;
		_half_size = size / 2.0f;
		_max_velocity = _model.getWidth() / 55.0f;
	}
	
	public float getX() {
		return _x;
	}
	
	public float getY() {
		return _y;
	}
	
	public float getSize() {
		return _size;
	}
	
	public void setPos(float x, float y) {
		_x = x;
		_y = y;
	}
	
	public void setVelocity(float vx, float vy) {
		_vx = vx;
		_vy = vy;
	}
	
	public void update() {
		_x += _vx;
		if (_x >= _model.getWidth() - _half_size || _x <= 0.0f + _half_size) {
			_model.onScore(_x, _y);
			return;
		}
		_y += _vy;
		if (_y >= _model.getHeight() - _half_size || _y <= 0.0f + _half_size) {
			_vy *= -1.0f;
			//Don't stick at the border
			if (Math.abs(_vy) - 0.01f < 0.0f) {
				_vy += _vy / Math.abs(_vy) * _model.getHeight() * 0.01f; 
			}
		}
		if (_x < (_model.getWidth() / 2.0f) && checkPaddleBallOverlap(_model.getPaddle(0))) {
			_x = _model.getPaddle(0).getX() + (_model.getPaddle(0).getWidth() / 2.0f) + _half_size;
			collide(_model.getPaddle(0));
		} else if (_x > (_model.getWidth() / 2.0f) && checkPaddleBallOverlap(_model.getPaddle(1))) {
			_x = _model.getPaddle(1).getX() - (_model.getPaddle(1).getWidth() / 2.0f) - _half_size;
			collide(_model.getPaddle(1));
		}
	}
	
	protected boolean checkPaddleBallOverlap(PongPaddleModel paddle) {
		//X-Axis
		float paddle_x = paddle.getX();
		float paddle_half_width = paddle.getWidth() / 2.0f;
		float x1 = paddle_x - paddle_half_width;
		float x2 = paddle_x + paddle_half_width;
		float x3 = _x - _half_size;
		float x4 = _x + _half_size;
		boolean x_overlap = (x1 <= x3 && x3 <= x2) || (x1 <= x4 && x4 <= x2) || (x3 <= x1 && x2 <= x4);
		if (!x_overlap) {
			return false;
		}
		//Y-Axis
		float paddle_y = paddle.getY();
		float paddle_half_height = paddle.getHeight() / 2.0f;
		float y1 = paddle_y - paddle_half_height;
		float y2 = paddle_y + paddle_half_height;
		float y3 = _y - _half_size;
		float y4 = _y + _half_size;
		boolean y_overlap = (y1 <= y3 && y3 <= y2) || (y1 <= y4 && y4 <= y2) || (y3 <= y1 && y2 <= y4);
		if (!y_overlap) {
			return false;
		}
		return true;
	}
	
	protected void collide(PongPaddleModel paddle) {
		_model.onChange(new PointF(paddle.getX(), paddle.getY()));
		if (Math.abs(_vx) < _max_velocity * _model.getSettings().getSpeed()) {
			//increase speed a little bit...
			_vx *= -1.0f - (_model.getWidth() * _model.getSettings().getSpeed() * (0.0004f + (float)Math.random() * 0.0002f));
		} else {
			_vx *= -1.0f;
		}
		//change direction depending on paddle movement
		_vy += paddle.getVelocityY();
	}
}
