//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.model;


public class PongPaddleModel {
	protected PongGameModel _model;
	//Position
	protected float _x;
	protected float _y;
	//Velocity
	protected float _vx;
	protected float _vy;
	//Size
	protected float _width;
	protected float _height;
	protected float _half_width;
	protected float _half_height;
	
	public PongPaddleModel(PongGameModel model, float x, float y, float width, float height) {
		_model = model;
		_x = x;
		_y = y;
		_vx = 0.0f;
		_vy = 0.0f;
		_width = width;
		_half_width = _width / 2.0f;
		_height = height;
		_half_height = _height / 2.0f;
	}
	
	public float getX() {
		return _x;
	}
	
	public void setX(float x) {
		_x = x;
	}
	
	public float getY() {
		return _y;
	}
	
	public void setY(float y) {
		_y = y;
	}
	
	public float getWidth() {
		return _width;
	}
	
	public float getHeight() {
		return _height;
	}
	
	public float getVelocityX() {
		return _vx;
	}
	
	public float getVelocityY() {
		return _vy;
	}
	
	public void setVelocity(float vx, float vy) {
		_vx = vx;
		_vy = vy;
	}
	
	public void update() {
		_x += _vx;
		_y += _vy;
		//TODO Add range checks for x if needed
		if (_y + _half_height > _model.getHeight()) {
			_y = _model.getHeight() - _half_height;
		} else if (_y - _half_height < 0.0f) {
			_y = _half_height;
		}
		_vx *= 0.25f;
		_vy *= 0.25f;
	}
	
	public void accelerate(float fx, float fy) {
		float acceleration = _model.getHeight() * 0.01f;
		_vx += acceleration * Math.max(Math.min(2.0f, fx), -2.0);
		_vy += acceleration * Math.max(Math.min(2.0f, fy), -2.0);
	}
}
