//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.model;


public class PongPlayerHuman extends PongPlayer {

	protected PongGameModel _model;
	protected PongInputSystem _input;
	protected int _paddle_index;
	protected int _controller_type;
	
	public PongPlayerHuman(PongGameModel model, int paddle_index, int controller_type) {
		_model = model;
		_input = (PongInputSystem) model.getInputSystem();
		_paddle_index = paddle_index;
		_controller_type = controller_type;
	}
	
	@Override
	public void update() {
		float movement = 0.0f;
		switch (_controller_type) {
		case PongSettings.INPUT_KEYBOARD:
			movement = _input.getKeyboardMovement() * 1.25f;
			break;
		case PongSettings.INPUT_TOUCHSCREEN:
			float reference_y = _model.getPaddle(_paddle_index).getY();
			movement = _input.getTouchMovement(reference_y, _paddle_index) * 1.25f;
			break;
		case PongSettings.INPUT_TRACKBALL:
			movement = _input.getTrackballMovement();
			break;
		}
		_model.getPaddle(_paddle_index).accelerate(0.0f, movement);
	}
}
