//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.model;

import android.content.SharedPreferences;

public class PongSettings {

	protected SharedPreferences _preferences;
	
	public static final int INPUT_TOUCHSCREEN = 0;
	public static final int INPUT_TRACKBALL = 1;
	public static final int INPUT_KEYBOARD = 2;
	public static final int INPUT_AI = 3;
	
	public static final String GRAPHICS_CLASSIC = "classic";
	public static final String GRAPHICS_COLORED = "colored";
	public static final String GRAPHICS_TEXTURED = "textured";
	
	public static final int MAX_POINTS = 5;
	public static final float SPEED = 1.0f;
	
	public PongSettings(SharedPreferences preferences) {
		_preferences = preferences;
	}
	
	public boolean isDebug() {
		return false;
	}
	
	public int getPlayer1Input() {
		String value = _preferences.getString("player1_input", String.valueOf(INPUT_TOUCHSCREEN));
		return Integer.valueOf(value).intValue();
	}
	
	public int getPlayer2Input() {
		String value = _preferences.getString("player2_input", String.valueOf(INPUT_AI));
		return Integer.valueOf(value).intValue();
	}
	
	public String getGraphicsStyle() {
		return _preferences.getString("graphics_style", GRAPHICS_CLASSIC);
	}
	
	public boolean playSound() {
		return _preferences.getBoolean("sound", true);
	}
	
	public boolean hardAIPlayer() {
		return _preferences.getBoolean("hard_ai", false);
	}
	
	public int getMaxPoints() {
		String value = _preferences.getString("max_points", String.valueOf(MAX_POINTS));
		return Integer.valueOf(value).intValue();
	}
	
	public float getSpeed() {
		String value = _preferences.getString("speed", String.valueOf(SPEED));
		return Float.valueOf(value).floatValue();
	}
}
