//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.renderer;

import java.nio.FloatBuffer;
import java.util.ArrayList;
import java.util.List;

import javax.microedition.khronos.opengles.GL11;

public class MeshInfo {
	protected int _offset;
	protected float[] _vertices;
	
	protected static final int VERTEX_COMPONENTS = 8; // 3 Position, 3 Normal, 2 UV
	protected static final int VERTEX_SIZE = VERTEX_COMPONENTS * 4; // 4 Bytes per float
	protected static final int POSITION_OFFSET = 0;
	protected static final int NORMAL_OFFSET = 12;
	protected static final int TEXCOORD_OFFSET = 24;
	
	protected static List<MeshInfo> _instances = new ArrayList<MeshInfo>();
	protected static final int SQUARE = 0;
	protected static final int BIGGER_SQUARE = 1;
	
	static {
		//Square
		new MeshInfo(new float[] {
				//vertices          //normals         //tex coords
				-0.5f, -0.5f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f,
				 0.5f, -0.5f, 0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 0.0f,
				 0.5f,  0.5f, 0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f,
				-0.5f, -0.5f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f,
				 0.5f,  0.5f, 0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f,
				-0.5f,  0.5f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 1.0f,
		});
		//Bigger Square
		new MeshInfo(new float[] {
				//vertices          //normals         //tex coords
				-1.0f, -1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f,
				 1.0f, -1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 0.0f,
				 1.0f,  1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f,
				-1.0f, -1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f,
				 1.0f,  1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f,
				-1.0f,  1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 1.0f,
		});
	}
	
	private MeshInfo(float[] vertices) {
		_vertices = vertices;
		MeshInfo._instances.add(this);
	}
	
	public int getVertexCount() {
		return _vertices.length / VERTEX_COMPONENTS;
	}
	
	public void loadToBuffer(GL11 gl, FloatBuffer buffer) {
		_offset = buffer.position() / VERTEX_COMPONENTS;
		buffer.put(_vertices);
	}
	
	public void draw(GL11 gl) {
		gl.glDrawArrays(GL11.GL_TRIANGLES, _offset, getVertexCount());
	}
	
	public static MeshInfo getFromID(int id) {
		return MeshInfo._instances.get(id);
	}
}
