//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.renderer;

import javax.microedition.khronos.opengles.GL11;

import aschiefer.bouncy.R;
import aschiefer.bouncy.model.PongGameModel;
import aschiefer.bouncy.model.PongSettings;

public class PongBackgroundRenderer {
	
	protected PongGameModel _model;
	protected ResourceManager _resources;
	protected float[] _color = { 1.0f, 1.0f, 1.0f, 1.0f };
	protected boolean _textured = false;
	protected int _mesh_id;
	
	protected static float RENDER_LAYER = 0.3f;
	
	public PongBackgroundRenderer(PongGameModel model, ResourceManager resources) {
		_model = model;
		_resources = resources;
		String style = _model.getSettings().getGraphicsStyle();
		if (style.equals(PongSettings.GRAPHICS_CLASSIC)) {
			_color = new float[] { 1.0f, 1.0f, 1.0f, 1.0f };
		} else if (style.equals(PongSettings.GRAPHICS_COLORED)) {
			_color = new float[] { 1.0f, 1.0f, 0.0f, 0.5f };
		} else if (style.equals(PongSettings.GRAPHICS_TEXTURED)) {
			_color = new float[] { 1.0f, 1.0f, 1.0f, 1.0f };
			_textured = true;
		}
		_mesh_id = MeshInfo.SQUARE;
	}

	public void setup(GL11 gl) {
		_resources.requestMesh(_mesh_id);
		_resources.getTextureID(gl, R.raw.background);
	}
	
	public void draw(GL11 gl) {
		gl.glTranslatef(_model.getWidth() / 2.0f, _model.getHeight() / 2.0f, -RENDER_LAYER);
		gl.glColor4f(_color[0], _color[1], _color[2], _color[3]);
		if (_textured) {
			gl.glEnable(GL11.GL_TEXTURE_2D);
			gl.glBindTexture(GL11.GL_TEXTURE_2D, _resources.getTextureID(gl, R.raw.background));
			gl.glScalef(_model.getWidth(), _model.getHeight(), 1.0f);
		} else {
			gl.glScalef(_model.getWidth() * 0.005f, _model.getHeight(), 1.0f);
		}
		_resources.drawMesh(gl, _mesh_id);
		if (_textured) {
			gl.glDisable(GL11.GL_TEXTURE_2D);
		}
	}
}
