//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.renderer;


import javax.microedition.khronos.opengles.GL11;

import aschiefer.bouncy.R;
import aschiefer.bouncy.model.PongBallModel;
import aschiefer.bouncy.model.PongGameModel;
import aschiefer.bouncy.model.PongSettings;

public class PongBallRenderer {
	
	protected PongGameModel _model;
	protected ResourceManager _resources;
	protected PongBallModel _ball;
	protected boolean _colored = false;
	protected boolean _textured = false;
	protected int _mesh_id;
	
	protected static float RENDER_LAYER = 0.1f;
	
	public PongBallRenderer(PongGameModel model, ResourceManager resources) {
		_model = model;
		_resources = resources;
		_ball = _model.getBall();
		_colored = _model.getSettings().getGraphicsStyle().equals(PongSettings.GRAPHICS_COLORED);
		_textured = _colored || _model.getSettings().getGraphicsStyle().equals(PongSettings.GRAPHICS_TEXTURED);
		if (_textured) {
			_mesh_id = MeshInfo.BIGGER_SQUARE;
		} else {
			_mesh_id = MeshInfo.SQUARE;
		}
	}

	public void setup(GL11 gl) {
		_resources.requestMesh(MeshInfo.SQUARE);
		_resources.requestMesh(MeshInfo.BIGGER_SQUARE);
		_resources.getTextureID(gl, R.raw.ball);
		_resources.getTextureID(gl, R.raw.ball_colored);
	}
	
	public void draw(GL11 gl) {
		gl.glTranslatef(_ball.getX(), _ball.getY(), -RENDER_LAYER);
		gl.glScalef(_ball.getSize(), _ball.getSize(), 1.0f);
		if (_colored) {
			float f = _ball.getX() / _model.getWidth();
			gl.glColor4f(0.0f, (float)Math.sin(f), (float)Math.sin((1.0f-f) * Math.PI/2.0f), 1.0f);
		} else {
			gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		}
		if (_textured) {
			gl.glEnable(GL11.GL_TEXTURE_2D);
			if (_colored) {
				gl.glBindTexture(GL11.GL_TEXTURE_2D, _resources.getTextureID(gl, R.raw.ball_colored));
			} else {
				gl.glBindTexture(GL11.GL_TEXTURE_2D, _resources.getTextureID(gl, R.raw.ball));
			}
		}
		_resources.drawMesh(gl, _mesh_id);
		if (_textured) {
			gl.glDisable(GL11.GL_TEXTURE_2D);
		}
	}
}
