//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.renderer;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.microedition.khronos.opengles.GL11;

public class VertexBuffer {
	
	protected int _vbo; 
	
	public VertexBuffer(GL11 gl) {
		int[] vbo_buffer = new int[1];
		gl.glGenBuffers(1, vbo_buffer, 0);
		_vbo = vbo_buffer[0];
		bind(gl);
	}
	
	public void addObjects(GL11 gl, MeshInfo[] objects) {
		int vertex_count = 0;
		for (MeshInfo obj : objects) {
			vertex_count += obj.getVertexCount();
		}
		ByteBuffer vbb = ByteBuffer.allocateDirect(vertex_count * MeshInfo.VERTEX_SIZE);
		vbb.order(ByteOrder.nativeOrder());
		FloatBuffer vertices = vbb.asFloatBuffer();
		for (MeshInfo obj : objects) {
			obj.loadToBuffer(gl, vertices);
		}
		vbb.position(0);
		gl.glBufferData(GL11.GL_ARRAY_BUFFER, vbb.capacity(), vbb, GL11.GL_STATIC_DRAW);
	}
	
	public void bind(GL11 gl) {
		gl.glEnableClientState(GL11.GL_VERTEX_ARRAY);
		gl.glEnableClientState(GL11.GL_NORMAL_ARRAY);
		gl.glEnableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
		
		gl.glBindBuffer(GL11.GL_ARRAY_BUFFER, _vbo);
		
		gl.glVertexPointer(3, GL11.GL_FLOAT, 32, MeshInfo.POSITION_OFFSET);
		gl.glNormalPointer(GL11.GL_FLOAT, 32, MeshInfo.NORMAL_OFFSET);
		gl.glTexCoordPointer(2, GL11.GL_FLOAT, 32, MeshInfo.TEXCOORD_OFFSET);
	}
}
